%% Set parameters

h1 = 600;
h2 = 600;
H = 4500;
h3 = H - h1 - h2;
W = 4000000;
L = 6500000;
g = 9.8;
f = 1*10^-4;
b = 1.5*10^-11;
at = 2*10^-4;
as = 7*10^-4;
dT = -20;
dS = -1.5;
um = 1*10^-2;

c = 1;
dyT = 2/L*(dT - as/at*dS);
dzT = -2*c/(h1 + h2)*(dT - as/at*dS);

a1y = -(h1*(h2 + h3))/(2*H);
b1y = -(h2*(h2 + 2*h3))/(2*H);
a2y = (h1^2)/(2*H);
b2y = -(h2*(h3 - h1))/(2*H);

a1z = - b/(2*f)*h1*a1y;
b1z = - b/(2*f)*h1*b1y;
a2z = - b/(2*f)*(2*h1*a1y + h2*a2y);
b2z = - b/(2*f)*(2*h1*b1y + h2*b2y);

a1 = at*g/f * (a1y*dyT + a1z*dzT) - um;
b1 = at*g/f * (b1y*dyT + b1z*dzT);
a2 = at*g/f * (a2y*dyT + a2z*dzT);
b2 = at*g/f * (b2y*dyT + b2z*dzT) - um;

year = 60*60*24*365;

af = 0;

%% Set parameters extended model (background overturning)

vm = 0.1*10^(-2);
wm = -0.17*10^(-6)/5;

vms = vm * b/f

w11 = wm * 2/h1
w12 = wm * 4/h1;
w22 = wm * 2/h2;

%% Scale Model

a1s = a1/W * year;
b1s = b1/W * year;
a2s = a2/W * year;
b2s = b2/W * year;

vs = vms * year;
w11s = w11 * year;
w12s = w12 * year;
w22s = w22 * year;


%% Set grid

dx = 0.0005;
xgrid = transpose(0:dx:1-dx);
Nx = 1/dx;

dt = 0.0005;
tmax = 1000;
time = 0:dt:tmax;
Nt = int64(tmax/dt+1);

% Initialise temperature vector
T1 = zeros(1,Nt);   T2 = zeros(1,Nt);
T1e = zeros(1,Nt);  T2e = zeros(1,Nt);

%% Create Initial Conditions

T1t0 = exp(-(xgrid-0.5).^2./(2*(0.15)^2));
T2t0 = zeros(Nx,1);

% Standard
T1t = T1t0;     T1tn = zeros(Nx,1);
T2t = T2t0;     T2tn = zeros(Nx,1);

% Extended
T1et = T1t0;    T1etn = zeros(Nx,1);
T2et = T2t0;    T2etn = zeros(Nx,1);


%% Plot initial conditions

figure
plot(xgrid, T1t0, '-r', xgrid, T2t0, '-.b', 'Linewidth', 2.5)
ax = gca; % current axes
ax.FontSize = 20;
ylim([0 1.1])
xlabel('x','FontSize',24)
ylabel('Temperature ({^\circ}C)','FontSize',24)
legend('T_1(0,x)','T_2(0,x)') 
grid on

% saveas(gcf,'AMO_IC_run.png')


%% Solve Sevellec-Huck model and extended model

tic
for t = 1:Nt
    
    % Shift the full vector
    T1xp = circshift(T1t,-1);       T1xp(Nx) = -T1xp(Nx);
    T2xp = circshift(T2t,-1);       T2xp(Nx) = -T2xp(Nx);
    
    T1exp = circshift(T1et,-1);     T1exp(Nx) = -T1exp(Nx);
    T2exp = circshift(T2et,-1);     T2exp(Nx) = -T2exp(Nx);

    % Standard
    T1tn = T1t + dt * ( a1s * (T1xp - T1t)/dx + b1s * (T2xp - T2t)/dx ...
            - af*T1t );
    T2tn = T2t + dt * ( b2s * (T2xp - T2t)/dx + a2s * (T1xp - T1t)/dx ...
            - af*T2t );
    
    T1(t) = T1tn(1);    T2(t) = T2tn(1);    % Boundary conditions
    T1t = T1tn;         T2t = T2tn;         % Update
        
    % Extended
    T1etn = T1et + dt * ( a1s * (T1exp - T1et)/dx + b1s * (T2exp - T2et)/dx ...
            - (vs + w11s) * T1et + w12s * T2et - af*T1et );
    T2etn = T2et + dt * ( b2s * (T2exp - T2et)/dx + a2s * (T1exp - T1et)/dx ...
            - (vs + w22s) * T2et - af*T2et );
    
    T1e(t) = T1etn(1);  T2e(t) = T2etn(1);  % Boundary conditions
    T1et = T1etn;       T2et = T2etn;       % Update
    
end
toc


%% Plot evolution T1, T2

figure
plot(time, T1,'-r',time, T2, '-.b', 'Linewidth', 2.5)

ax = gca; % current axes
ax.FontSize = 20;

xlim([ 0 200 ])
ylim([-1. 1.])

legend('T_1', 'T_2 ','Location','northeast','Orientation','horizontal')
xlabel('Time (year)','FontSize',24)
ylabel('Temperature ({^\circ}C)','FontSize',24)
grid on

% saveas(gcf, 'AMO_run_new.png')


% With extended background circulation
figure
plot(time, T1,'-r',time, T2, '-.b', 'Linewidth', 2.5)
hold on
plot(time, T1e,'-k',time, T2e, '-.c', 'Linewidth', 2.5)

ax = gca; % current axes
ax.FontSize = 20;

xlim([ 400 700 ])
ylim([-1. 1.])

legend('T_1', 'T_2 ','T_1^B ', 'T_2^B ','Location','northeast','Orientation','horizontal')
xlabel('Time (year)','FontSize',24)
ylabel('Temperature ({^\circ}C)','FontSize',24)
grid on

% saveas(gcf,'AMO_background_new.png')

%% Get the Evolution of the Stream Function

phisc = 0.1;
phi1 = at*g/f * gradient(T1,dx)/W /phisc * H/W*year;
phi1(end-1:end) = NaN;

psisc = 0.1;
psi1 = -b/f * at*g/f * T1 /psisc * H/W*year;
psi1(end-1:end) = NaN;


figure
plot(time, phi1, 'b', time, psi1, 'k', time, T1,'r', 'Linewidth', 2.5)

ax = gca; % current axes
ax.FontSize = 20;

xlim([120 200])
ylim([-1.1 1.1])

legend('\partial_z v (0.1 s^{-1})', '\partial_z u (0.1 s^{-1})','T_1 (^\circC)', 'Location','northeast','Orientation','horizontal')

xlabel('Time (year)','FontSize',24)
ylabel(' ','FontSize',24)

grid on

% saveas(gcf,'AMO_Stream.png')
