function [xt,t,xend] = DDE_solver(f,hist,tspan,h,tau1,tau2)
% Solving initial-value problems for DDEs of form:
% dx/dt=f(t,x(t),x(t-tau1),x(t-tau2),par) 
%   1D time input and n-D x input to n-D x output
%   Trapezoidal 2nd order
%
%   f - model function
%   hist - initial history, must be of size [n-D,max(m1,m2)]
%   tspan - [start time, end time]
%   h - time step
%   tau1, tau2 - delay values

N = round((tspan(2)-tspan(1))/h);
m1 = round(tau1/abs(h));
m2 = round(tau2/abs(h));
M = size(hist,1);
xt = NaN(M,N+1);
t = NaN(N+1,1);
t(1) = tspan(1);
xt(:,1) = hist(:,end);
if m2 < m1
    xtau1 = hist(:,1);
    xtau2 = hist(:,1+m1-m2);
    for n = 1:m2-1
        t0 = t(n);
        x0 = xt(:,n);
        k = f(t0,x0,xtau1,xtau2);
        x_temp = x0 + h.*k;
        t0 = tspan(1)+ n*h;
        t(n+1) = t0;
        xtau1 = hist(:,n+1);
        xtau2 = hist(:,n+1+m1-m2);
        xt(:,n+1) = x0 + h./2.*(k+f(t0,x_temp,xtau1,xtau2));
    end
    for n = m2:m1-1
        t0 = t(n);
        x0 = xt(:,n);
        k = f(t0,x0,xtau1,xtau2);
        x_temp = x0 + h.*k;
        t0 = tspan(1)+ n*h;
        t(n+1) = t0;
        xtau1 = hist(:,n+1);
        xtau2 = xt(:,n-m2+1);
        xt(:,n+1) = x0 + h./2.*(k+f(t0,x_temp,xtau1,xtau2));
    end
    for n = m1:N
        t0 = t(n);
        x0 = xt(:,n);
        k = f(t0,x0,xtau1,xtau2);
        x_temp = x0 + h.*k;
        t0 = tspan(1)+ n*h;
        t(n+1) = t0;
        xtau1 = xt(:,n-m1+1);
        xtau2 = xt(:,n-m2+1);
        xt(:,n+1) = x0 + h./2.*(k+f(t0,x_temp,xtau1,xtau2));
    end
    if nargout > 1
        xend = xt(:,end);
    end
elseif m1 < m2
    xtau1 = hist(:,1+m2-m1);
    xtau2 = hist(:,1);
    for n = 1:m1-1
        t0 = t(n);
        x0 = xt(:,n);
        k = f(t0,x0,xtau1,xtau2);
        x_temp = x0 + h.*k;
        t0 = tspan(1)+ n*h;
        t(n+1) = t0;
        xtau1 = hist(:,n+1+m2-m1);
        xtau2 = hist(:,n+1);
        xt(:,n+1) = x0 + h./2.*(k+f(t0,x_temp,xtau1,xtau2));
    end
    for n = m1:m2-1
        t0 = t(n);
        x0 = xt(:,n);
        k = f(t0,x0,xtau1,xtau2);
        x_temp = x0 + h.*k;
        t0 = tspan(1)+ n*h;
        t(n+1) = t0;
        xtau1 = xt(:,n-m1+1);
        xtau2 = hist(:,n+1);
        xt(:,n+1) = x0 + h./2.*(k+f(t0,x_temp,xtau1,xtau2));
    end
    for n = m2:N
        t0 = t(n);
        x0 = xt(:,n);
        k = f(t0,x0,xtau1,xtau2);
        x_temp = x0 + h.*k;
        t0 = tspan(1)+ n*h;
        t(n+1) = t0;
        xtau1 = xt(:,n-m1+1);
        xtau2 = xt(:,n-m2+1);
        xt(:,n+1) = x0 + h./2.*(k+f(t0,x_temp,xtau1,xtau2));
    end
    if nargout > 1
        xend = xt(:,end);
    end
else 
    fprintf('\nError: Delays are equal\n\n')
end
end