%% This script can be used to produce Figures 7 and 9 from the manuscript
%  "A Delay Equation Model for the Atlantic Multidecadal Oscillation"
%   Falkena, Quinn, Sieber, and Dijkstra, 2020
%
%  To run this script, you will need the accompanying functions in your path:
%  AMO_MoC.m, DDE_solver.m, PSD_discrete.m
%
%  You will also need the open-source package chebfun:
%  https://www.chebfun.org 
%  T. A Driscoll, N. Hale, and L. N. Trefethen. Chebfun Guide. 
%  Pafnuty Publications, 2014

clear;
close all;

%% PDE parameters

c = 1;

h1 = 600;
h2 = 600;
h3 = 3300;
H = 4500;
g = 9.8;
f = 1e-4;
beta = 1.5e-11;
aT = 2e-4;
aS = 7e-4;
ubar = 1e-2;
deltaT = -20;
deltaS = -1.5;
L = 6500000;

dzT = -2*c/(h1+h2)*(deltaT-aS/aT*deltaS);
dyT = 2/L*(deltaT-aS/aT*deltaS);

timescale = 60*60*24*365;
lengthscale = 4000000;

%% Difference eq. parameters

a1 = -(ubar+aT*g/(2*H*f)*(h1*(h2+h3))*dyT...
    -aT*g*beta/(4*H*f^2)*(h1^2*(h2+h3))*dzT)*timescale/lengthscale;
b1 = -(aT*g/(2*H*f)*(h2*(h2+2*h3))*dyT...
    -aT*g*beta/(4*H*f^2)*(h1*h2*(h2+2*h3))*dzT)*timescale/lengthscale;
a2 = -(aT*g/(2*H*f)*(-h1^2)*dyT...
    -aT*g*beta/(2*H*f^2)*(h1^2*(h2/2+h3))*dzT)*timescale/lengthscale;
b2 = -(ubar+aT*g/(2*H*f)*(h2*(h3-h1))*dyT...
    -aT*g*beta/(2*H*f^2)*(2*h1*h2*h3+h2^2/2*(h1+h3))*dzT)*timescale/lengthscale;

lambda1 = 1/2*(a1+b2 + sqrt((a1+b2)^2-4*a1*b2+4*a2*b1));
lambda2 = 1/2*(a1+b2 - sqrt((a1+b2)^2-4*a1*b2+4*a2*b1));

tau1 = 1/lambda1;
tau2 = 1/lambda2;

c1 = [(lambda2-a1)./(lambda1-lambda2),...
   (lambda1-a1).*(lambda2-a1)./(a2*(lambda1-lambda2));
   -a2./(lambda1-lambda2),...
   -(lambda1-a1)./(lambda1-lambda2)];

c2 = [-(lambda1-a1)./(lambda1-lambda2),...
   -(lambda1-a1).*(lambda2-a1)./(a2*(lambda1-lambda2));
   a2./(lambda1-lambda2),...
   (lambda2-a1)./(lambda1-lambda2)];

%% PDE solution for history

%Set gridsize and time
h = 0.0005;
N = round(1/h);
dt = h;
tmax = round(max(tau1,tau2),2);
time = 0:dt:tmax+dt;
Nt = tmax/dt;

a1N = a1*N;
b1N = b1*N;
a2N = a2*N;
b2N = b2*N;
N2 = 2*N;
Nm = N-1;

%Create matrix system for discretized PDE
Di = repmat([-a1N -b2N],1,N);
DiM1 = repmat([-a2N 0],1,N);
DiP1 = repmat([-b1N a2N],1,N);
DiP2 = repmat([a1N b2N],1,Nm);
DiP3 = repmat([b1N 0],1,Nm);

DiM1(end) = []; DiP1(end) = []; DiP3(end) = [];

Mat0 = diag(Di) + diag(DiM1,-1) + diag(DiP1,1) + diag(DiP2,2) + diag(DiP3,3);
Mat = sparse(Mat0);

Mat(N2-1:N2,1:2) = [-a1N, -b1N; -a2N, -b2N];


% Guassian initial profile
cen = 0.5; sig = 0.15; amp = 1;
IC = zeros(2*N,1);
for x = 1:N
    IC(x*2-1) = amp * exp(-(x/(N) - cen)^2/(2*sig^2));
end

% Solution to discretized PDE for t = [-tau,0]
[Tode, tode] = ode45(@(t,v) Mat*v, [0 tmax+dt],IC);

T = transpose(interp1(tode, Tode, time));

% Retain time evolution of T1 and T2 at x=0
T_init = T(1:2,:);

%% DDE model
% this section produces Figure 7

eps = h;
par = [c, eps];
h = dt;
t1 = round(tau1*1/h)+1;
t2 = round(tau2*1/h);

[X_DDE,t_DDE,~] = DDE_solver(@(t,x,xlag1,xlag2)AMO_mydde(t,x,xlag1,xlag2,par),...
    T_init,[0,200],h,t1*h,t2*h);

figure;
hold on;
plot([-1*fliplr(time),t_DDE'],[T_init(1,:),X_DDE(1,:)],'r','linewidth',2)
plot([-1*fliplr(time),t_DDE'],[T_init(2,:),X_DDE(2,:)],'b-.','linewidth',2)
xlabel('Time (year)')
ylabel('Temperature (^\circC)')
set(gca,'fontsize',16)
xlim([0 200])
box on;
grid on;


freq = linspace(0,0.2,500)';
w = 2.*pi.*freq;
Twind = 200;
PSD_DDE = PSD_discrete(w,h*100,Twind,X_DDE(:,1:100:end)');

PSD_DDE(:,1) = PSD_DDE(:,1)/max(PSD_DDE(:,1));
PSD_DDE(:,2) = PSD_DDE(:,2)/max(PSD_DDE(:,2));

figure; hold on;
plot(freq,PSD_DDE(:,1),'r','linewidth',2)
plot(freq,PSD_DDE(:,2),'b-.','linewidth',2)
xlabel('frequency (years^{-1})')
ylabel('spectral density')
set(gca,'fontsize',20)
box on;
grid on;


%% Eigenfunctions of PDE model

PDE = chebop(0, 1);
PDE.op = @(x,T1,T2) [a1*diff(T1) + b1*diff(T2); a2*diff(T1) + b2*diff(T2)];
PDE.bc = @(x,T1,T2)[feval(T1,0)+feval(T1,1);...
                    feval(T2,0)+feval(T2,1)];
[V1_PDE, lam1_PDE] = eigs(PDE,20,'sm');
[V2_PDE, lam2_PDE] = eigs(PDE,50,'sm');

%% Eigenfunctions of DDE model (MoC)
% set up as an eigenvalue problem for a linear operator L
% the operator itself is just Lu=u'
%
% The DDE is in the boundary condition:
% 0=T1'(0)+T1(0)-c1_11*T1(-tau1)-c1_12*T2(-tau1)-c2_11*T1(-tau2)-c2_12*T2(-tau2)
% 0=T2'(0)+T2(0)-c1_21*T1(-tau1)-c1_22*T2(-tau1)-c2_21*T1(-tau2)-c2_22*T2(-tau2)

Eps = 1./linspace(20,2000,100);
V_MoC_Eps = cell(100,1);
lam_MoC_Eps = cell(100,1);

i = 0;

for eps = Eps

i = i + 1;
    
DDE_MoC =chebop(@(t,T1,T2)[diff(T1);diff(T2)]);
DDE_MoC.domain = [-tau2, 0];
DDE_MoC.bc = @(t,T1,T2)[feval(diff(T1),0)-1/eps*(-feval(T1,0)+c1(1,1)*feval(T1,-tau1)+c1(1,2)*feval(T2,-tau1)+c2(1,1)*feval(T1,-tau2)+c2(1,2)*feval(T2,-tau2));...
    feval(diff(T2),0)-1/eps*(-feval(T2,0)+c1(2,1)*feval(T1,-tau1)+c1(2,2)*feval(T2,-tau1)+c2(2,1)*feval(T1,-tau2)+c2(2,2)*feval(T2,-tau2))];
[V_MoC, lam_MoC] = eigs(DDE_MoC,50,'sm');

V_MoC_Eps{i} = V_MoC;
lam_MoC_Eps{i} = lam_MoC;

end

%% Eigenfunctions of DDE model (MZ)
% set up as an eigenvalue problem for a linear operator L
% the operator itself is just Lu=u'
%
% The DDE is in the boundary condition:
% 0=T1'(0)+T1(0)-c1_11*T1(-tau1)-c1_12*T2(-tau1)-c2_11*T1(-tau2)-c2_12*T2(-tau2)
% 0=T2'(0)+T2(0)-c1_21*T1(-tau1)-c1_22*T2(-tau1)-c2_21*T1(-tau2)-c2_22*T2(-tau2)

lp = 1/2*(a1+b2+sqrt(a1^2+b2^2-2*a1*b2+4*a2*b1));
lm = 1/2*(a1+b2-sqrt(a1^2+b2^2-2*a1*b2+4*a2*b1));

wp = 1/(2*a2)*(a1-b2+sqrt(a1^2+b2^2-2*a1*b2+4*a2*b1));
wm = 1/(2*a2)*(a1-b2-sqrt(a1^2+b2^2-2*a1*b2+4*a2*b1));

A1p = (a1*wp+b1)*(-a1+wm*a2)/(wp-wm);
A1m = -(a1*wm+b1)*(-a1+wp*a2)/(wp-wm);
B1p = (a1*wp+b1)*(-b1+wm*b2)/(wp-wm);
B1m = -(a1*wm+b1)*(-b1+wp*b2)/(wp-wm);
A2p = (a2*wp+b2)*(-a1+wm*a2)/(wp-wm);
A2m = -(a2*wm+b2)*(-a1+wp*a2)/(wp-wm);
B2p = (a2*wp+b2)*(-b1+wm*b2)/(wp-wm);
B2m = -(a2*wm+b2)*(-b1+wp*b2)/(wp-wm);

V_MZ_Eps = cell(100,1);
lam_MZ_Eps = cell(100,1);

i = 0;

for eps = Eps
 
i = i + 1;
    
DDE_MZ =chebop(@(t,T1,T2)[diff(T1);diff(T2)]);
DDE_MZ.domain = [-tau2, 0];
DDE_MZ.bc = @(t,T1,T2)...
   [feval(diff(T1),0)+1/eps*(a1*feval(T1,0)+b1*feval(T2,0)-A1p/lp*feval(T1,-tau1)-B1p/lp*feval(T2,-tau1)-A1m/lm*feval(T1,-tau2)-B1m/lm*feval(T2,-tau2));...
    feval(diff(T2),0)+1/eps*(a2*feval(T1,0)+b2*feval(T2,0)-A2p/lp*feval(T1,-tau1)-B2p/lp*feval(T2,-tau1)-A2m/lm*feval(T1,-tau2)-B2m/lm*feval(T2,-tau2))];
[V_MZ, lam_MZ] = eigs(DDE_MZ,50,'sm');

V_MZ_Eps{i} = V_MZ;
lam_MZ_Eps{i} = lam_MZ;

end

%% Eigenfunctions of discretized PDE model

%Set gridsize and time
Ns = linspace(20,2000,100);
V_PDE_disc_Ns = cell(100,1);
lam_PDE_disc_Ns = cell(100,1);

i = 0;

for N = Ns
    
i = i+1;
    
a1N = a1*N;
b1N = b1*N;
a2N = a2*N;
b2N = b2*N;
N2 = 2*N;
Nm = N-1;

%Create matrix system
Di = repmat([-a1N -b2N],1,N);
DiM1 = repmat([-a2N 0],1,N);
DiP1 = repmat([-b1N a2N],1,N);
DiP2 = repmat([a1N b2N],1,Nm);
DiP3 = repmat([b1N 0],1,Nm);

DiM1(end) = []; DiP1(end) = []; DiP3(end) = [];

Mat_BC0 = diag(Di) + diag(DiM1,-1) + diag(DiP1,1) + diag(DiP2,2) + diag(DiP3,3);
Mat_BC = sparse(Mat_BC0);

Mat_BC(N2-1:N2,1:2) = [-a1N, -b1N; -a2N, -b2N];

if N < 50
    [V_PDE_disc, lam_PDE_disc] = eigs(Mat_BC,20,'sm');
else
    [V_PDE_disc, lam_PDE_disc] = eigs(Mat_BC,50,'sm');
end

V_PDE_disc_Ns{i} = V_PDE_disc;
lam_PDE_disc_Ns{i} = lam_PDE_disc;

end


%% Plotting eigenvectors
% this section produces Figure 9a

nt=N;
dom=linspace(-tau2,0,nt);
dom_PDE=linspace(0,1,nt);
dom01=dom/tau2+1; 

vals_PDE = feval(V1_PDE(:,1),dom_PDE);
vals_MoC = feval(V_MoC(:,1),dom);
vals_MZ = feval(V_MZ(:,1),dom);

vals_PDE = {vals_PDE{1}./vals_PDE{1}(1),vals_PDE{2}./vals_PDE{1}(1)};
vals_MoC = {vals_MoC{1}./vals_MoC{1}(1),vals_MoC{2}./vals_MoC{1}(1)};
vals_MZ = {vals_MZ{1}./vals_MZ{1}(1),vals_MZ{2}./vals_MZ{1}(1)};

vals_PDE_disc = V_PDE_disc./V_PDE_disc(1,1);

figure; hold on;
p1 = plot(dom_PDE,real(vals_PDE{1}), 'b','linewidth',2);
p2 = plot(dom_PDE,real(vals_PDE{2}), 'r','linewidth',2);
axis([0 1 -1 1])
grid on,
legend([p1,p2],'V_1','V_2')
set(gca, 'FontSize',20)
xlabel('x');
ylabel('real(V)');
box on

%% Plotting errors with increasing N and eps
% this section produces Figure 9b

disc_PDE_BC_error = NaN(100,1);
MoC_BC_error = NaN(100,1);
MZ_BC_error = NaN(100,1);

for i = 1:100
    V_PDE_disc = V_PDE_disc_Ns{i};
    vals_PDE_disc = V_PDE_disc./V_PDE_disc(1,1);
    disc_PDE_BC_error(i) = abs(vals_PDE_disc(end-1,1)+vals_PDE_disc(1,1));
    
    V_MoC = V_MoC_Eps{i};
    vals_MoC = feval(V_MoC(:,1),dom);
    vals_MoC = {vals_MoC{1}./vals_MoC{1}(1),vals_MoC{2}./vals_MoC{1}(1)};
    MoC_BC_error(i) = abs(vals_MoC{1}(end)+vals_MoC{1}(1));
    
    V_MZ = V_MZ_Eps{i};
    vals_MZ = feval(V_MZ(:,1),dom);
    vals_MZ = {vals_MZ{1}./vals_MZ{1}(1),vals_MZ{2}./vals_MZ{1}(1)};
    MZ_BC_error(i) = abs(vals_MZ{1}(end)+vals_MZ{1}(1));
    
end

figure;
subplot(3,1,1)
plot(Ns,disc_PDE_BC_error,'linewidth',2)
set(gca,'Fontsize',20,'xscale','log','yscale','log')
xlim([20 2000])
ylim([0 0.16])
ylabel('error_{disc PDE}')
xlabel('N')
grid on
subplot(3,1,2)
plot(Eps,MoC_BC_error,'linewidth',2)
set(gca,'xdir','reverse','Fontsize',20,'xscale','log','yscale','log')
xlim([5*10^-4 5*10^-2])
ylim([0 6*10^-3])
ylabel('error_{MoC}')
xlabel('\epsilon')
grid on
subplot(3,1,3)
plot(Eps,MZ_BC_error,'linewidth',2)
set(gca,'xdir','reverse','Fontsize',20,'xscale','log','yscale','log')
xlim([5*10^-4 5*10^-2])
ylim([0 0.16])
ylabel('error_{MZ}')
xlabel('\epsilon')
grid on

